@@ -9,6 +9,7 @@
 #ifndef XTENSOR_R_CONTAINER_HPP
 #define XTENSOR_R_CONTAINER_HPP
 
+#include <complex>
 #include <functional>
 #include <numeric>
 
@@ -17,23 +18,72 @@
 
 #include "xtl/xsequence.hpp"
 
+#include "xtensor_r_config.hpp"
+
 #include <Rcpp.h>
 #include <RcppCommon.h>
 
+#include "rcpp_extensions.hpp"
+
+// Fake type for R logicals
+struct rlogical {};
+
+// This handles the conversion of xtensor back to R type
+// When an xtensor with a T of rlogical is seen, the retrieved rtype
+// is LGLSXP (even though it is stored internally as an int in xtensor)
+namespace Rcpp
+{
+    namespace traits
+    {
+        template<> struct r_sexptype_traits<rlogical>
+        {
+            static constexpr int rtype = LGLSXP;
+        };
+    }
+}
+
 namespace xt
 {
 
+    // R stores logicals as int32. This ensures that when a logical is seen,
+    // the internal xtensor storage type uses an int.
+    namespace r_detail
+    {
+        template <class T>
+        struct get_underlying_value_type_r
+        {
+            using type = T;
+        };
+
+        template <>
+        struct get_underlying_value_type_r<rlogical>
+        {
+            using type = int;
+        };
+    }
+
     namespace detail
     {
         inline xbuffer_adaptor<int*> r_shape_to_buffer_adaptor(SEXP exp)
         {
-            if (Rf_isNull(Rf_getAttrib(exp, R_DimSymbol)))
+            SEXP shape_sexp;
+            SEXP dim = Rf_getAttrib(exp, R_DimSymbol);
+
+            if (Rf_isNull(dim))
             {
-                auto d = Rcpp::IntegerVector::create(Rf_length(exp));
-                Rf_setAttrib(exp, R_DimSymbol, d);
+                // This is a 0D scalar
+                if (Rf_xlength(exp) == 1)
+                {
+                    return xbuffer_adaptor<int*>(nullptr, 0);
+                }
+
+                shape_sexp = Rcpp::IntegerVector::create(Rf_length(exp));
+            }
+            else
+            {
+                shape_sexp = dim;
             }
 
-            SEXP shape_sexp = Rf_getAttrib(exp, R_DimSymbol);
             std::size_t n = (std::size_t)Rf_xlength(shape_sexp);
             return xbuffer_adaptor<int*>(
                 Rcpp::internal::r_vector_start<INTSXP>(shape_sexp), n);
@@ -41,22 +91,44 @@ namespace xt
 
         inline xbuffer_adaptor<int*> r_shape_to_buffer_adaptor(SEXP exp, std::size_t n)
         {
-            if (Rf_isNull(Rf_getAttrib(exp, R_DimSymbol)))
+            SEXP shape_sexp;
+            SEXP dim = Rf_getAttrib(exp, R_DimSymbol);
+
+            if (n == 0)
             {
-                auto d = Rcpp::IntegerVector::create(Rf_length(exp));
-                Rf_setAttrib(exp, R_DimSymbol, d);
+                return xbuffer_adaptor<int*>(nullptr, 0);
+            }
+
+            if (Rf_isNull(dim))
+            {
+                shape_sexp = Rcpp::IntegerVector::create(Rf_length(exp));
+            }
+            else 
+            {
+                shape_sexp = dim;
             }
 
-            SEXP shape_sexp = Rf_getAttrib(exp, R_DimSymbol);
             if (n != (std::size_t)Rf_xlength(shape_sexp))
             {
-                throw std::runtime_error("Could not convert shape. Dimensions don't match.");
+                throw std::runtime_error("Could not convert shape for rtensor. Dimensions don't match.");
             }
             return xbuffer_adaptor<int*>(
                 Rcpp::internal::r_vector_start<INTSXP>(shape_sexp), n);
         }
-    }
 
+        template <int SXP>
+        inline void check_coercion(SEXP exp)
+        {
+        #if XTENSOR_WARN_ON_COERCE
+            if (TYPEOF(exp) != SXP)
+            {
+                Rcpp::warning("Coerced object from '%s' to '%s'. Avoid for speed & in-place operations.",
+                    Rf_type2char(TYPEOF(exp)),
+                    Rf_type2char(SXP));
+            }
+        #endif
+        }
+    }
 
     /**
      * @class rcontainer
@@ -67,16 +139,20 @@ namespace xt
      *
      * @tparam D The derived type, i.e. the inheriting class for which rcontainer
      *           provides the interface.
+     * @tparam SP The Rcpp storage policy, defaults to Rcpp::PreserveStorage
      */
-    template <class D>
-    class rcontainer : public xcontainer<D>
+    template <class D, template <class> class SP = Rcpp::PreserveStorage>
+    class rcontainer : public xcontainer<D>, public SP<D>
     {
     public:
 
         using derived_type = D;
 
+        using rstorage = SP<D>;
+
         using base_type = xcontainer<D>;
         using inner_types = xcontainer_inner_types<D>;
+        using r_type = typename inner_types::r_type;
         using storage_type = typename inner_types::storage_type;
         using value_type = typename storage_type::value_type;
         using reference = typename storage_type::reference;
@@ -86,6 +162,14 @@ namespace xt
         using size_type = typename storage_type::size_type;
         using difference_type = typename storage_type::difference_type;
 
+#ifndef XTENSOR_R_ALLOW_REINTERPRETATION
+        static_assert(xtl::disjunction<std::is_same<r_type, int32_t>,
+                                       std::is_same<r_type, double>,
+                                       std::is_same<r_type, Rbyte>,
+                                       std::is_same<r_type, rlogical>,
+                                       std::is_same<r_type, std::complex<double>>>::value == true,
+                      "R containers can only be of type rlogical, int, double, std::complex<double>.");
+#endif
         using shape_type = typename inner_types::shape_type;
         using strides_type = typename inner_types::strides_type;
         using backstrides_type = typename inner_types::backstrides_type;
@@ -104,22 +188,20 @@ namespace xt
         static constexpr bool contiguous_layout = true;
 
         template <class S = shape_type>
-        void resize(S&& shape);
+        void resize(S&& shape, bool force = false);
 
         template <class S = shape_type>
         void reshape(S&& shape);
 
         layout_type layout() const;
 
-        operator SEXP() const;
-        void set_sexp(SEXP exp);
+        // explicitly forward data against ambiguity with Rcpp::Storage::data;
+        using base_type::data;
 
     protected:
 
-        rcontainer();
-        ~rcontainer();
-
-        rcontainer(SEXP exp);
+        rcontainer() = default;
+        ~rcontainer() = default;
 
         rcontainer(const rcontainer&) = default;
         rcontainer& operator=(const rcontainer&) = default;
@@ -131,54 +213,21 @@ namespace xt
         derived_type& derived_cast() & noexcept;
         const derived_type& derived_cast() const & noexcept;
         derived_type derived_cast() && noexcept;
-
-    private:
-
-        SEXP m_sexp;
-        bool m_owned;
     };
 
 
-    template <class D>
-    rcontainer<D>::rcontainer()
-        : m_sexp(R_NilValue), m_owned(true)
-    {
-    }
-
-    template <class D>
-    rcontainer<D>::rcontainer(SEXP exp)
-        : m_sexp(R_NilValue), m_owned(false)
-    {
-        m_sexp = Rcpp::Rcpp_ReplaceObject(m_sexp, exp);
-    }
-
-    template <class D>
-    rcontainer<D>::~rcontainer()
-    {
-        if (m_owned)
-        {
-            Rcpp::Rcpp_ReleaseObject(m_sexp);
-            m_sexp = R_NilValue;
-        }
-    }
-
-    template <class D>
-    void rcontainer<D>::set_sexp(SEXP exp)
-    {
-        m_sexp = Rcpp::Rcpp_ReplaceObject(m_sexp, exp);
-    }
-
     /**
      * Resizes the container.
      * @param shape the new shape
      */
-    template <class D>
+    template <class D, template <class> class SP>
     template <class S>
-    inline void rcontainer<D>::resize(S&& shape)
+    inline void rcontainer<D, SP>::resize(S&& shape, bool force)
     {
-        if (shape.size() != this->dimension() || !std::equal(std::begin(shape), std::end(shape), this->shape().cbegin()))
+        // if SEXP not initialized, it will be NULL (e.g. in constructor)
+        if (Rf_isNull(*this) || shape.size() != this->dimension() || !std::equal(std::begin(shape), std::end(shape), this->shape().cbegin()) || force)
         {
-            derived_type tmp(xtl::forward_sequence<shape_type>(shape));
+            derived_type tmp(xtl::forward_sequence<shape_type, S>(shape));
             *static_cast<derived_type*>(this) = std::move(tmp);
         }
     }
@@ -187,9 +236,9 @@ namespace xt
      * Reshapes the container.
      * @param shape the new shape
      */
-    template <class D>
+    template <class D, template <class> class SP>
     template <class S>
-    inline void rcontainer<D>::reshape(S&& shape)
+    inline void rcontainer<D, SP>::reshape(S&& shape)
     {
         if (compute_size(shape) != this->size())
         {
@@ -199,37 +248,31 @@ namespace xt
         if (shape.size() != this->dimension() || !std::equal(std::begin(shape), std::end(shape), this->shape().cbegin()))
         {
             auto tmp_shape = Rcpp::IntegerVector(std::begin(shape), std::end(shape));
-            Rf_setAttrib(m_sexp, R_DimSymbol, SEXP(tmp_shape));
-            this->derived_cast().set_shape();
+            Rf_setAttrib(rstorage::get__(), R_DimSymbol, SEXP(tmp_shape));
+            this->derived_cast().update_shape_and_strides();
         }
     }
 
-    template <class D>
-    inline layout_type rcontainer<D>::layout() const
+    template <class D, template <class> class SP>
+    inline layout_type rcontainer<D, SP>::layout() const
     {
         return layout_type::column_major;
     }
 
-    template <class D>
-    inline rcontainer<D>::operator SEXP() const
-    {
-        return m_sexp;
-    }
-
-    template <class D>
-    inline auto rcontainer<D>::derived_cast() & noexcept -> derived_type&
+    template <class D, template <class> class SP>
+    inline auto rcontainer<D, SP>::derived_cast() & noexcept -> derived_type&
     {
         return *static_cast<derived_type*>(this);
     }
 
-    template <class D>
-    inline auto rcontainer<D>::derived_cast() const & noexcept -> const derived_type&
+    template <class D, template <class> class SP>
+    inline auto rcontainer<D, SP>::derived_cast() const & noexcept -> const derived_type&
     {
         return *static_cast<const derived_type*>(this);
     }
 
-    template <class D>
-    inline auto rcontainer<D>::derived_cast() && noexcept -> derived_type
+    template <class D, template <class> class SP>
+    inline auto rcontainer<D, SP>::derived_cast() && noexcept -> derived_type
     {
         return *static_cast<derived_type*>(this);
     }
