@@ -9,7 +9,8 @@
 #ifndef XTENSOR_AXIS_ITERATOR_HPP
 #define XTENSOR_AXIS_ITERATOR_HPP
 
-#include "xtl/xclosure.hpp"
+#include <xtl/xclosure.hpp>
+
 #include "xview.hpp"
 
 namespace xt
