@@ -38,6 +38,11 @@ namespace xt
         using inner_strides_type = inner_shape_type;
         using inner_backstrides_type = inner_shape_type;
 
+#if defined(__GNUC__) && !defined(__clang__) && __GNUC__ == 8
+        constexpr static auto random_instantiation_var_for_gcc8_data_iface = has_data_interface<xdynamic_view<CT, S, L, FST>>::value;
+        constexpr static auto random_instantiation_var_for_gcc8_has_strides = has_strides<xdynamic_view<CT, S, L, FST>>::value;
+#endif
+
         // TODO: implement efficient stepper specific to the dynamic_view
         using const_stepper = xindexed_stepper<const xdynamic_view<CT, S, L, FST>, true>;
         using stepper = xindexed_stepper<xdynamic_view<CT, S, L, FST>, false>;
