@@ -103,6 +103,15 @@ namespace xt
         template <class E>
         derived_type& modulus_assign(const xexpression<E>&);
 
+        template <class E>
+        derived_type& bit_and_assign(const xexpression<E>&);
+
+        template <class E>
+        derived_type& bit_or_assign(const xexpression<E>&);
+
+        template <class E>
+        derived_type& bit_xor_assign(const xexpression<E>&);
+
     protected:
 
         xsemantic_base() = default;
@@ -164,21 +173,8 @@ namespace xt
         derived_type& operator=(const xexpression<E>&);
     };
 
-    namespace detail
-    {
-        template <class E>
-        struct has_container_semantics_impl : std::is_base_of<xcontainer_semantic<std::decay_t<E>>, std::decay_t<E>>
-        {
-        };
-
-        template <class E>
-        struct has_container_semantics_impl<xcontainer_semantic<E>> : std::true_type
-        {
-        };
-    }
-
     template <class E>
-    using has_container_semantics = detail::has_container_semantics_impl<E>;
+    using has_container_semantics = is_crtp_base_of<xcontainer_semantic, E>;
 
     template <class E, class R = void>
     using enable_xcontainer_semantics = typename std::enable_if<has_container_semantics<E>::value, R>::type;
@@ -231,21 +227,8 @@ namespace xt
         derived_type& operator=(const xexpression<E>&);
     };
 
-    namespace detail
-    {
-        template <class E>
-        struct has_view_semantics_impl : std::is_base_of<xview_semantic<std::decay_t<E>>, std::decay_t<E>>
-        {
-        };
-
-        template <class E>
-        struct has_view_semantics_impl<xview_semantic<E>> : std::true_type
-        {
-        };
-    }
-
     template <class E>
-    using has_view_semantics = detail::has_view_semantics_impl<E>;
+    using has_view_semantics = is_crtp_base_of<xview_semantic, E>;
 
     template <class E, class R = void>
     using enable_xview_semantics = typename std::enable_if<has_view_semantics<E>::value, R>::type;
@@ -535,6 +518,45 @@ namespace xt
         return this->derived_cast().computed_assign(this->derived_cast() % e.derived_cast());
     }
 
+    /**
+     * Computes the bitwise and of \c e to \c *this. Ensures no temporary
+     * will be used to perform the assignment.
+     * @param e the xexpression to add.
+     * @return a reference to \c *this.
+     */
+    template <class D>
+    template <class E>
+    inline auto xsemantic_base<D>::bit_and_assign(const xexpression<E>& e) -> derived_type&
+    {
+        return this->derived_cast().computed_assign(this->derived_cast() & e.derived_cast());
+    }
+
+    /**
+     * Computes the bitwise or of \c e to \c *this. Ensures no temporary
+     * will be used to perform the assignment.
+     * @param e the xexpression to add.
+     * @return a reference to \c *this.
+     */
+    template <class D>
+    template <class E>
+    inline auto xsemantic_base<D>::bit_or_assign(const xexpression<E>& e) -> derived_type&
+    {
+        return this->derived_cast().computed_assign(this->derived_cast() | e.derived_cast());
+    }
+
+    /**
+     * Computes the bitwise xor of \c e to \c *this. Ensures no temporary
+     * will be used to perform the assignment.
+     * @param e the xexpression to add.
+     * @return a reference to \c *this.
+     */
+    template <class D>
+    template <class E>
+    inline auto xsemantic_base<D>::bit_xor_assign(const xexpression<E>& e) -> derived_type&
+    {
+        return this->derived_cast().computed_assign(this->derived_cast() ^ e.derived_cast());
+    }
+
     template <class D>
     template <class E>
     inline auto xsemantic_base<D>::operator=(const xexpression<E>& e) -> derived_type&
@@ -605,12 +627,32 @@ namespace xt
         return this->derived_cast();
     }
 
+    namespace detail
+    {
+        template <class F>
+        bool get_rhs_triviality(const F&)
+        {
+            return true;
+        }
+
+        template <class F, class R, class... CT>
+        bool get_rhs_triviality(const xfunction<F, R, CT...>& rhs)
+        {
+            using index_type = xindex_type_t<typename xfunction<F, R, CT...>::shape_type>;
+            using size_type = typename index_type::size_type;
+            size_type size = rhs.dimension();
+            index_type shape = xtl::make_sequence<index_type>(size, size_type(0));
+            bool trivial_broadcast = rhs.broadcast_shape(shape, true);
+            return trivial_broadcast;
+        }
+    }
+
     template <class D>
     template <class E>
     inline auto xview_semantic<D>::assign_xexpression(const xexpression<E>& e) -> derived_type&
     {
         xt::assert_compatible_shape(*this, e);
-        xt::assign_data(*this, e, false);
+        xt::assign_data(*this, e, detail::get_rhs_triviality(e.derived_cast()));
         return this->derived_cast();
     }
 
@@ -619,7 +661,7 @@ namespace xt
     inline auto xview_semantic<D>::computed_assign(const xexpression<E>& e) -> derived_type&
     {
         xt::assert_compatible_shape(*this, e);
-        xt::assign_data(*this, e, false);
+        xt::assign_data(*this, e, detail::get_rhs_triviality(e.derived_cast()));
         return this->derived_cast();
     }
 
@@ -628,26 +670,33 @@ namespace xt
     inline auto xview_semantic<D>::scalar_computed_assign(const E& e, F&& f) -> derived_type&
     {
         D& d = this->derived_cast();
-        std::transform(d.begin(), d.end(), d.begin(),
-                       [e, &f](const auto& v) { return f(v, e); });
+
+        using size_type = typename D::size_type;
+        auto dst = d.begin();
+        for (size_type i = d.size(); i > 0; --i)
+        {
+            *dst = f(*dst, e);
+            ++dst;
+        }
         return this->derived_cast();
     }
 
     template <class D>
     template <class E>
-    inline auto xview_semantic<D>::operator=(const xexpression<E>& e) -> derived_type&
+    inline auto xview_semantic<D>::operator=(const xexpression<E>& rhs) -> derived_type&
     {
-        bool cond = (e.derived_cast().shape().size() == this->derived_cast().dimension()) &&
+        bool cond = (rhs.derived_cast().shape().size() == this->derived_cast().dimension()) &&
             std::equal(this->derived_cast().shape().begin(),
                        this->derived_cast().shape().end(),
-                       e.derived_cast().shape().begin());
+                       rhs.derived_cast().shape().begin());
+
         if (!cond)
         {
-            base_type::operator=(broadcast(e.derived_cast(), this->derived_cast().shape()));
+            base_type::operator=(broadcast(rhs.derived_cast(), this->derived_cast().shape()));
         }
         else
         {
-            base_type::operator=(e);
+            base_type::operator=(rhs);
         }
         return this->derived_cast();
     }
